﻿package com.everydayflash.futbolowo {
	
	import five3D.display.DynamicText3D;
	import five3D.display.Graphics3D;
	import five3D.display.Shape3D;
	import five3D.display.Sprite3D;
	import flash.display.CapsStyle;
	import flash.display.Sprite;

	import flash.events.Event;
	import gs.TweenLite;
	import fl.motion.easing.Sine;
	
	import five3D.utils.Drawing;

	public class Stadium extends Sprite3D {
		
		private var ground:Shape3D;
		private var mainTribune:Sprite3D;
		private var leftGoal:Shape3D;
		private var rightGoal:Shape3D;
		
		private var leftFlag:Shape3D;
		private var rightFlag:Shape3D;
		
		private var scoreBoard:Sprite3D;
		
		private var gW:Number = 400;
		private var gH:Number = 250;
		
		private var fieldMargin:Number = gW / 20;
		
		private var gX:Number = gW * -0.5;
		private var gY:Number = gH * -0.5;
		
		private var mW:Number = gW * 0.75;
		private var mH:Number = gH / 16;
		private var mA:Number = 90;
		
		private var top:Number = gY + fieldMargin;
		private var bottom:Number =  gY + gH - fieldMargin;	
		private var left:Number = gX + fieldMargin;
		private var right:Number =  gX + gW - fieldMargin;
		
		private var penaltyHeight:Number = gH / 3;
		private var penaltyWidth:Number = gW / 10;
		
		private var goalWidth:Number = penaltyHeight - 16;
		private var goalHeight:Number = penaltyWidth - 10;
		
		private var hM:Number = 0;
		private var vM:Number = 0;
		
		private var tribuneColor:uint = 0xcccccc;
		private var goalColor:uint = 0xffcc00;
		private var groundColor:uint = 0x7cc92e;
		private var linesColor:uint = 0xffffff;

		private var angle:Number = 0;
		
		private var actors:Actors;
		
		private var scoreText:Sprite3D;
		private var whyText:Sprite3D;
		private var createText:Sprite3D;
		
		public function Stadium(actors:Actors) {
			super();
			visible = false;
			this.actors = actors;
			createGround();
			//createFlags();
			createMainTribune();
			createGoals();
		}
		
		private function createGround():void {
			ground = new Shape3D();
			drawGround(ground.graphics3D);
			drawFieldLines(ground.graphics3D);
			addChild(ground);
		}
		
		public function animateGround():void {
			whyText = actors.whyWait;
			whyText.x = gX + 53 - 50;
			whyText.y = gY + 50;
			whyText.alpha = 0;
			addChild(whyText);
			
			createText = actors.createOne;
			createText.x = gX + 78 + 50;
			createText.y = gY + 51;
			createText.alpha = 0;
			addChild(createText);
			
			TweenLite.to(whyText, .2, { 
				x:gX + 53, alpha:1, ease:Sine.easeInOut
			} );
			
			TweenLite.to(createText, .2, { 
				x:gX + 78, alpha:1, delay:.4, ease:Sine.easeInOut
			} );
		}
		
		private function drawGround(gg:Graphics3D):void {
			gg.beginFill(linesColor, 1);
			gg.drawRoundRect(gX, gY, gW, gH, 90, 90);
			gg.endFill();
			
			gg.beginFill(groundColor, 1);
			gg.drawRoundRect(gX+20, gY+20, gW-40, gH-40, 20, 20);
			gg.endFill();
		}
		
		private function drawFieldLines(gg:Graphics3D):void {
			gg.lineStyle(3, linesColor, 1, true, "normal", CapsStyle.NONE);
			
			gg.moveTo(vM, top-10);
			gg.lineTo(vM, bottom+10);
			
			gg.moveTo(left, hM - penaltyHeight / 2);
			gg.lineTo(left + penaltyWidth, hM - penaltyHeight / 2);
			gg.lineTo(left + penaltyWidth, hM + penaltyHeight / 2);
			gg.lineTo(left, hM + penaltyHeight / 2);
			
			gg.moveTo(right, hM - penaltyHeight / 2);
			gg.lineTo(right - penaltyWidth, hM - penaltyHeight / 2);
			gg.lineTo(right - penaltyWidth, hM + penaltyHeight / 2);
			gg.lineTo(right, hM + penaltyHeight / 2);
			
			gg.drawCircle(vM, hM, penaltyHeight/2);
		}
		
		public function createMainTribune():void {
			mainTribune = new Sprite3D();
			var mainTribuneSeats:Shape3D = new Shape3D()
			drawMainTribune(mainTribuneSeats.graphics3D);
			mainTribuneSeats.rotationX = mA;
			
			mainTribune.addChild(mainTribuneSeats);
			mainTribune.x = gX + 50;
			mainTribune.y = gY;
			addChild(mainTribune);
		}
		
		private function drawMainTribune(gg:Graphics3D):void {
			gg.beginFill(tribuneColor, 1);
			gg.drawRect(0, -mH, mW, mH);
			gg.endFill();
		}
		
		private function createGoals():void {
			
			leftGoal = new Shape3D();
			leftGoal.x = left;
			leftGoal.y = hM - goalWidth / 2;
			var lg:Graphics3D = leftGoal.graphics3D;
			
			lg.lineStyle(4, goalColor, 1, true, "normal", CapsStyle.NONE);
			lg.moveTo(0, 0);
			lg.lineTo(goalHeight, 0);
			lg.lineTo(goalHeight, goalWidth);
			lg.lineTo(0, goalWidth);
			
			leftGoal.rotationY = 90;
			addChild(leftGoal);
			
			rightGoal = new Shape3D();
			rightGoal.x = right;
			rightGoal.y = hM - goalWidth / 2;
			var rg:Graphics3D = rightGoal.graphics3D;
			
			rg.lineStyle(4, goalColor, 1, true, "normal", CapsStyle.NONE);
			rg.moveTo(0, 0);
			rg.lineTo(goalHeight * -1, 0);
			rg.lineTo(goalHeight * -1, goalWidth);
			rg.lineTo(0, goalWidth);
			
			rightGoal.rotationY = -90;
			addChild(rightGoal);
		}
		
		public function createFlags():void {
			leftFlag = new Shape3D();
			var lf:Graphics3D = leftFlag.graphics3D;
			lf.beginFill(Data.STAND, 1);
			lf.drawRect(0,0,3,70);
			lf.endFill();
			
			lf.beginFill(Data.YELLOW, 1);
			lf.drawRect(3, 55, 20, 15);
			lf.endFill();
			
			leftFlag.y = gY + gH;
			leftFlag.x = gX + 50;
			leftFlag.rotationX -= 90;
			addChild(leftFlag);

			rightFlag = new Shape3D();
			var rf:Graphics3D = rightFlag.graphics3D;
			rf.beginFill(Data.STAND, 1);
			rf.drawRect(0,0,3,70);
			rf.endFill();
			
			rf.beginFill(Data.LIGHT_GREEN, 1);
			rf.drawRect(3, 55, 20, 15);
			rf.endFill();
			
			rightFlag.y = gY + gH;
			rightFlag.x = gX + gW - 50;
			rightFlag.rotationX -= 90;
			addChild(rightFlag);
			
			createScoreboard();
			
			addEventListener(Event.ENTER_FRAME, waveFlags);
		}
		
		private function waveFlags(e:Event):void {
			angle += 10;
			leftFlag.rotationY = Math.sin(angle / 180 * Math.PI) * 10;
			rightFlag.rotationY = Math.sin(angle / 180 * Math.PI) * 10;

			//scoreText.alpha = .3 + ((Math.sin(angle * 10 / 180 * Math.PI) + 1) * .5) * .6;
		}
		
		public function createScoreboard():void {
			var sbW:Number = gW / 2.5;
			
			scoreBoard = new Sprite3D();

			var sbFrame:Shape3D = new Shape3D();
			var sfg:Graphics3D = sbFrame.graphics3D;
			sfg.beginFill(Data.STAND, 1);
			sfg.drawRect(10, 0, 4, 30);
			sfg.drawRect(sbW-14, 0, 4, 30);
			sfg.drawRoundRect(0, 30, sbW, 50, 5, 5);
			sfg.endFill();
			
			sfg.beginFill(Data.STAND_INSIDE, 1);
			sfg.drawRoundRect(3, 33, sbW-6, 44, 3, 3);
			sfg.endFill();

			scoreBoard.addChild(sbFrame);

			scoreBoard.x = sbW * -0.5;
			scoreBoard.y = gY + gH;
			scoreBoard.rotationX -= 90;
			
			scoreText = Data.get3DText(Data.forFree);
			scoreText.rotationZ += 180;
			scoreText.x += sbW / 2 + 53;
			scoreText.y += 54;
			scoreBoard.addChild(scoreText);
			
			addChild(scoreBoard);
		}
		
		public function cleanUp():void {
			removeChild(whyText);
			removeChild(createText);
			//scoreBoard.removeChild(scoreText);
		}
	}
}




















